@@ -45,7 +45,7 @@ int _mosquitto_packet_handle(struct mosquitto *mosq)
 {
 	assert(mosq);
 
-	switch((mosq->core.in_packet.command)&0xF0){
+	switch((mosq->in_packet.command)&0xF0){
 		case PINGREQ:
 			return _mosquitto_handle_pingreq(mosq);
 		case PINGRESP:
@@ -59,7 +59,7 @@ int _mosquitto_packet_handle(struct mosquitto *mosq)
 		case PUBREC:
 			return _mosquitto_handle_pubrec(mosq);
 		case PUBREL:
-			return _mosquitto_handle_pubrel(mosq);
+			return _mosquitto_handle_pubrel(NULL, mosq);
 		case CONNACK:
 			return _mosquitto_handle_connack(mosq);
 		case SUBACK:
@@ -68,60 +68,11 @@ int _mosquitto_packet_handle(struct mosquitto *mosq)
 			return _mosquitto_handle_unsuback(mosq);
 		default:
 			/* If we don't recognise the command, return an error straight away. */
-			_mosquitto_log_printf(mosq, MOSQ_LOG_ERR, "Error: Unrecognised command %d\n", (mosq->core.in_packet.command)&0xF0);
+			_mosquitto_log_printf(mosq, MOSQ_LOG_ERR, "Error: Unrecognised command %d\n", (mosq->in_packet.command)&0xF0);
 			return MOSQ_ERR_PROTOCOL;
 	}
 }
 
-int _mosquitto_handle_pingreq(struct mosquitto *mosq)
-{
-	assert(mosq);
-#ifdef WITH_STRICT_PROTOCOL
-	if(mosq->core.in_packet.remaining_length != 0){
-		return MOSQ_ERR_PROTOCOL;
-	}
-#endif
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PINGREQ");
-	return _mosquitto_send_pingresp(mosq);
-}
-
-int _mosquitto_handle_pingresp(struct mosquitto *mosq)
-{
-	assert(mosq);
-#ifdef WITH_STRICT_PROTOCOL
-	if(mosq->core.in_packet.remaining_length != 0){
-		return MOSQ_ERR_PROTOCOL;
-	}
-#endif
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PINGRESP");
-	return MOSQ_ERR_SUCCESS;
-}
-
-int _mosquitto_handle_pubackcomp(struct mosquitto *mosq, const char *type)
-{
-	uint16_t mid;
-	int rc;
-
-	assert(mosq);
-#ifdef WITH_STRICT_PROTOCOL
-	if(mosq->core.in_packet.remaining_length != 2){
-		return MOSQ_ERR_PROTOCOL;
-	}
-#endif
-	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
-	if(rc) return rc;
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received %s (Mid: %d)", type, mid);
-
-	if(!_mosquitto_message_delete(mosq, mid, mosq_md_out)){
-		/* Only inform the client the message has been sent once. */
-		if(mosq->on_publish){
-			mosq->on_publish(mosq->obj, mid);
-		}
-	}
-
-	return MOSQ_ERR_SUCCESS;
-}
-
 int _mosquitto_handle_publish(struct mosquitto *mosq)
 {
 	uint8_t header;
@@ -133,14 +84,14 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 	message = _mosquitto_calloc(1, sizeof(struct mosquitto_message_all));
 	if(!message) return MOSQ_ERR_NOMEM;
 
-	header = mosq->core.in_packet.command;
+	header = mosq->in_packet.command;
 
 	message->direction = mosq_md_in;
 	message->dup = (header & 0x08)>>3;
 	message->msg.qos = (header & 0x06)>>1;
 	message->msg.retain = (header & 0x01);
 
-	rc = _mosquitto_read_string(&mosq->core.in_packet, &message->msg.topic);
+	rc = _mosquitto_read_string(&mosq->in_packet, &message->msg.topic);
 	if(rc){
 		_mosquitto_message_cleanup(&message);
 		return rc;
@@ -156,17 +107,17 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 	}
 
 	if(message->msg.qos > 0){
-		rc = _mosquitto_read_uint16(&mosq->core.in_packet, &message->msg.mid);
+		rc = _mosquitto_read_uint16(&mosq->in_packet, &message->msg.mid);
 		if(rc){
 			_mosquitto_message_cleanup(&message);
 			return rc;
 		}
 	}
 
-	message->msg.payloadlen = mosq->core.in_packet.remaining_length - mosq->core.in_packet.pos;
+	message->msg.payloadlen = mosq->in_packet.remaining_length - mosq->in_packet.pos;
 	if(message->msg.payloadlen){
 		message->msg.payload = _mosquitto_calloc(message->msg.payloadlen+1, sizeof(uint8_t));
-		_mosquitto_read_bytes(&mosq->core.in_packet, message->msg.payload, message->msg.payloadlen);
+		_mosquitto_read_bytes(&mosq->in_packet, message->msg.payload, message->msg.payloadlen);
 		if(rc){
 			_mosquitto_message_cleanup(&message);
 			return rc;
@@ -202,56 +153,3 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 	}
 }
 
-int _mosquitto_handle_pubrec(struct mosquitto *mosq)
-{
-	uint16_t mid;
-	int rc;
-
-	assert(mosq);
-#ifdef WITH_STRICT_PROTOCOL
-	if(mosq->core.in_packet.remaining_length != 2){
-		return MOSQ_ERR_PROTOCOL;
-	}
-#endif
-	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
-	if(rc) return rc;
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PUBREC (Mid: %d)", mid);
-
-	rc = _mosquitto_message_update(mosq, mid, mosq_md_out, mosq_ms_wait_pubcomp);
-	if(rc) return rc;
-	rc = _mosquitto_send_pubrel(mosq, mid, false);
-	if(rc) return rc;
-
-	return MOSQ_ERR_SUCCESS;
-}
-
-int _mosquitto_handle_pubrel(struct mosquitto *mosq)
-{
-	uint16_t mid;
-	struct mosquitto_message_all *message = NULL;
-	int rc;
-
-	assert(mosq);
-#ifdef WITH_STRICT_PROTOCOL
-	if(mosq->core.in_packet.remaining_length != 2){
-		return MOSQ_ERR_PROTOCOL;
-	}
-#endif
-	rc = _mosquitto_read_uint16(&mosq->core.in_packet, &mid);
-	if(rc) return rc;
-	_mosquitto_log_printf(mosq, MOSQ_LOG_DEBUG, "Received PUBREL (Mid: %d)", mid);
-
-	if(!_mosquitto_message_remove(mosq, mid, mosq_md_in, &message)){
-		/* Only pass the message on if we have removed it from the queue - this
-		 * prevents multiple callbacks for the same message. */
-		if(mosq->on_message){
-			mosq->on_message(mosq->obj, &message->msg);
-		}else{
-			_mosquitto_message_cleanup(&message);
-		}
-	}
-	rc = _mosquitto_send_pubcomp(mosq, mid);
-	if(rc) return rc;
-
-	return MOSQ_ERR_SUCCESS;
-}
