@@ -34,7 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 int _mosquitto_packet_alloc(struct _mosquitto_packet *packet);
 void _mosquitto_check_keepalive(struct mosquitto *mosq);
 int _mosquitto_fix_sub_topic(char **subtopic);
-uint16_t _mosquitto_mid_generate(struct _mosquitto_core *core);
+uint16_t _mosquitto_mid_generate(struct mosquitto *mosq);
 bool _mosquitto_wildcard_check(const char *str);
 
 #endif
