@@ -43,6 +43,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <mqtt3.h>
 #include <memory_mosq.h>
+#include <util_mosq.h>
 
 extern bool flag_reload;
 #ifdef WITH_PERSISTENCE
@@ -85,8 +86,8 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 		if(new_clients){
 			client_max = -1;
 			for(i=0; i<db->context_count; i++){
-				if(db->contexts[i] && db->contexts[i]->core.sock >= 0 && db->contexts[i]->core.sock > sock_max){
-					client_max = db->contexts[i]->core.sock;
+				if(db->contexts[i] && db->contexts[i]->sock >= 0 && db->contexts[i]->sock > sock_max){
+					client_max = db->contexts[i]->sock;
 				}
 			}
 			new_clients = 0;
@@ -101,7 +102,7 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 			pollfd_count = sock_max+1;
 			pollfds = _mosquitto_realloc(pollfds, sizeof(struct pollfd)*pollfd_count);
 			if(!pollfds){
-				mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
+				_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 				return MOSQ_ERR_NOMEM;
 			}
 		}
@@ -117,30 +118,30 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 		now = time(NULL);
 		for(i=0; i<db->context_count; i++){
 			if(db->contexts[i]){
-				if(db->contexts[i]->core.sock >= 0){
-					if(db->contexts[i]->core.sock > sock_max){
-						sock_max = db->contexts[i]->core.sock;
+				if(db->contexts[i]->sock >= 0){
+					if(db->contexts[i]->sock > sock_max){
+						sock_max = db->contexts[i]->sock;
 					}
 #ifdef WITH_BRIDGE
 					if(db->contexts[i]->bridge){
-						mqtt3_check_keepalive(db->contexts[i]);
+						_mosquitto_check_keepalive(db->contexts[i]);
 					}
 #endif
-					if(!(db->contexts[i]->core.keepalive) || now - db->contexts[i]->core.last_msg_in < (time_t)(db->contexts[i]->core.keepalive)*3/2){
+					if(!(db->contexts[i]->keepalive) || now - db->contexts[i]->last_msg_in < (time_t)(db->contexts[i]->keepalive)*3/2){
 						if(mqtt3_db_message_write(db->contexts[i]) == MOSQ_ERR_SUCCESS){
-							if(db->contexts[i]->core.sock < pollfd_count){
-								pollfds[db->contexts[i]->core.sock].fd = db->contexts[i]->core.sock;
-								pollfds[db->contexts[i]->core.sock].events = POLLIN;
-								pollfds[db->contexts[i]->core.sock].revents = 0;
-								if(db->contexts[i]->core.out_packet){
-									pollfds[db->contexts[i]->core.sock].events |= POLLOUT;
+							if(db->contexts[i]->sock < pollfd_count){
+								pollfds[db->contexts[i]->sock].fd = db->contexts[i]->sock;
+								pollfds[db->contexts[i]->sock].events = POLLIN;
+								pollfds[db->contexts[i]->sock].revents = 0;
+								if(db->contexts[i]->out_packet){
+									pollfds[db->contexts[i]->sock].events |= POLLOUT;
 								}
 							}
 						}else{
 							mqtt3_context_disconnect(db, i);
 						}
 					}else{
-						mqtt3_log_printf(MOSQ_LOG_NOTICE, "Client %s has exceeded timeout, disconnecting.", db->contexts[i]->core.id);
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client %s has exceeded timeout, disconnecting.", db->contexts[i]->id);
 						/* Client has exceeded keepalive*1.5 */
 						mqtt3_context_disconnect(db, i);
 					}
@@ -151,10 +152,10 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 						if(!db->contexts[i]->bridge->restart_t){
 							db->contexts[i]->bridge->restart_t = time(NULL)+30;
 							if(db->contexts[i]->bridge->notifications){
-								notification_topic_len = strlen(db->contexts[i]->core.id)+strlen("$SYS/broker/connection//state");
+								notification_topic_len = strlen(db->contexts[i]->id)+strlen("$SYS/broker/connection//state");
 								notification_topic = _mosquitto_malloc(sizeof(char)*(notification_topic_len+1));
 								if(notification_topic){
-									snprintf(notification_topic, notification_topic_len+1, "$SYS/broker/connection/%s/state", db->contexts[i]->core.id);
+									snprintf(notification_topic, notification_topic_len+1, "$SYS/broker/connection/%s/state", db->contexts[i]->id);
 									notification_payload[0] = '0';
 									notification_payload[1] = '\0';
 									mqtt3_db_messages_easy_queue(db, db->contexts[i], notification_topic, 1, 2, (uint8_t *)&notification_payload, 1);
@@ -169,7 +170,7 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 						}
 					}else{
 #endif
-						if(db->contexts[i]->core.clean_session == true){
+						if(db->contexts[i]->clean_session == true){
 							mqtt3_context_cleanup(db, db->contexts[i], true);
 							db->contexts[i] = NULL;
 						}
@@ -221,7 +222,7 @@ int mosquitto_main_loop(mosquitto_db *db, int *listensock, int listensock_count,
 		}
 #endif
 		if(flag_reload){
-			mqtt3_log_printf(MOSQ_LOG_INFO, "Reloading config.");
+			_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Reloading config.");
 			mqtt3_config_read(db->config, true);
 			mosquitto_security_cleanup(db);
 			mosquitto_security_init(db);
@@ -246,12 +247,12 @@ static void loop_handle_errors(mosquitto_db *db, struct pollfd *pollfds)
 	int i;
 
 	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && db->contexts[i]->core.sock >= 0){
-			if(pollfds[db->contexts[i]->core.sock].revents & POLLHUP){
-				if(db->contexts[i]->core.state != mosq_cs_disconnecting){
-					mqtt3_log_printf(MOSQ_LOG_NOTICE, "Socket error on client %s, disconnecting.", db->contexts[i]->core.id);
+		if(db->contexts[i] && db->contexts[i]->sock >= 0){
+			if(pollfds[db->contexts[i]->sock].revents & POLLHUP){
+				if(db->contexts[i]->state != mosq_cs_disconnecting){
+					_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Socket error on client %s, disconnecting.", db->contexts[i]->id);
 				}else{
-					mqtt3_log_printf(MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->core.id);
+					_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->id);
 				}
 				mqtt3_context_disconnect(db, i);
 			}
@@ -264,26 +265,26 @@ static void loop_handle_reads_writes(mosquitto_db *db, struct pollfd *pollfds)
 	int i;
 
 	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && db->contexts[i]->core.sock >= 0){
-			if(pollfds[db->contexts[i]->core.sock].revents & POLLOUT){
-				if(mqtt3_net_write(db->contexts[i])){
-					if(db->contexts[i]->core.state != mosq_cs_disconnecting){
-						mqtt3_log_printf(MOSQ_LOG_NOTICE, "Socket write error on client %s, disconnecting.", db->contexts[i]->core.id);
+		if(db->contexts[i] && db->contexts[i]->sock >= 0){
+			if(pollfds[db->contexts[i]->sock].revents & POLLOUT){
+				if(_mosquitto_packet_write(db->contexts[i])){
+					if(db->contexts[i]->state != mosq_cs_disconnecting){
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Socket write error on client %s, disconnecting.", db->contexts[i]->id);
 					}else{
-						mqtt3_log_printf(MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->core.id);
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->id);
 					}
 					/* Write error or other that means we should disconnect */
 					mqtt3_context_disconnect(db, i);
 				}
 			}
 		}
-		if(db->contexts[i] && db->contexts[i]->core.sock >= 0){
-			if(pollfds[db->contexts[i]->core.sock].revents & POLLIN){
-				if(mqtt3_net_read(db, i)){
-					if(db->contexts[i]->core.state != mosq_cs_disconnecting){
-						mqtt3_log_printf(MOSQ_LOG_NOTICE, "Socket read error on client %s, disconnecting.", db->contexts[i]->core.id);
+		if(db->contexts[i] && db->contexts[i]->sock >= 0){
+			if(pollfds[db->contexts[i]->sock].revents & POLLIN){
+				if(_mosquitto_packet_read(db, i)){
+					if(db->contexts[i]->state != mosq_cs_disconnecting){
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Socket read error on client %s, disconnecting.", db->contexts[i]->id);
 					}else{
-						mqtt3_log_printf(MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->core.id);
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client %s disconnected.", db->contexts[i]->id);
 					}
 					/* Read error or other that means we should disconnect */
 					mqtt3_context_disconnect(db, i);
